% descartes_kissing_circles
% "In geometry, Descartes' theorem states that
% for every four kissing, or mutually tangent, circles, the radii of the
% circles satisfy a certain quadratic equation. By solving this equation,
% one can construct a fourth circle tangent to three given, mutually
% tangent circles. The theorem is named after Ren Descartes, who stated it
% in 1643."
%
% This program applies Descartes's theorem iteratively to generate the
% Apollonian Gasket fractal.
%
% LAST UPDATED by Andy French July 2023

function descartes_kissing_circles

%Set input
starting_configuration = 'random three circles';
%starting_configuration = 'Pappus curves';

%Max radius for random circles or a parameter (0 < a < 1) for Pappas curves
a = 10;
%phi = (1+sqrt(5))/2; a = 1/phi;

%Colour scheme
colour_scheme = 'random';
colour_scheme = 'mondrian';

%Linewidths and edge colours for circles
lwidth = 0.3; edge_RGB = [0,0,0];

%Plot update every P circles
P = 100;

%Max number of circles to plot
C = 3000;

%

%Display starting message and start clock
clc; disp(' '); disp(' DESCARTES KISSING CIRCLES GENERATING THE APOLLONIAN GASKET ')
t0 = tic;

%Determine starting configuration of circle (x,y) centres, radii r, new
%kiss matrix k and circumcircle X,Y,R
if strcmp( starting_configuration,'Pappus curves')
    X = 1/2; Y = 0; R = 1/2; [x,y,r,k] = pappus(a,50);
    rmin = 1e-4; %Minimum radius for circles
    N = 20; %Iterations for determing additional circles   
else
    [x,y,r,X,Y,R,k] = random_three_circle_input;
    rmin = 0.02; %Minimum radius for circles
    N = 50; %Iterations for determing additional circles
end

%Determine all possible new circle kisses and generate new circles.
for n=1:N
    [x,y,r,k] = more_kissing_circles(x,y,r,X,Y,R,k,rmin);
end

%Reflect in x axis with Pappus input
if strcmp( starting_configuration,'Pappus curves')
    x = [x,x]; y = [y,-y]; r = [r,r];
end

%Sort circles by radius
[r,i] = sort(r, 'descend'); x = x(i); y =  y(i);

%

%Set up a blank figure window with a blank axes that can be added to with
%plot commands
fig = figure('renderer','opengl',...
    'units','normalized','position',[0.1,0.1,0.8,0.8],...
    'name','... Constructing Apollonius Gasket'); axes('NextPlot','add');

%Plot white circumcircle
circle( X,Y,R,[1,1,1],lwidth,edge_RGB  ); axis tight; axis equal; axis manual; axis off;

%Determine title and filename
if strcmp( starting_configuration,'Pappus curves')
    t = ['Apollonian gasket a=',num2str(a,3)];
    filename = ['Apollonian gasket a=',strrep(num2str(a,3),'.','p'),'.png'];
else
    t = ['Apollonian gasket r1=',num2str(r(1)),', r2=',num2str(r(2)),', r3=',num2str(r(3))];
    filename = [strrep(t,'.','p'),'.png'];
end
tit = title('... Plotting Apollonius Gasket','fontsize',18);

%Plot other circles, cycling through Nc x 3 array of RGB colours
[colours,Nc] = choose_colormap( colour_scheme );  j=1; c = 1;
for n=1:min([C,numel(x)])
    
    %Determine circle colour and plot circle
    if isempty(colours)
        RGB = rand(1,3);
    else
        RGB = colours(c,:);
    end
    circle( x(n),y(n),r(n),RGB,lwidth,edge_RGB  );
    
    %Update figure ever P circles
    if j==P; drawnow; j=1; end    
    %Update circle and colour counters
    j = j+1; c = c + 1; if c>Nc; c = 1; end
end

%Print figure and close
set(tit,'string',t,'fontsize',14); drawnow; 
print( gcf,filename,'-dpng','-r600' ); close(gcf);
disp( [' ... Gasket printed to a PNG file in ',num2str(toc(t0)),'s.']); disp(' ');

%%

%Random three circle input
function [x,y,r,X,Y,R,k] = random_three_circle_input

%Define four element vectors of the centre (x,y) and radius r of three
%(externally) mutually tangent circles. The radii are randomly chosen.
[x,y,r] = three_kissing_circles;

%Determine centre and radius of circle 4, which should kiss the other
%three, and a circumcircle of circles 1,2,3, of centre (X,Y) and radius R
[x(4),y(4),r(4),X,Y,R] = descartes( x(1),y(1),r(1), x(2),y(2),r(2), x(3),y(3), r(3) );

%Check that X,Y,R actually is a circumcircle - if not regenerate
while ( ( ( X-( x(4) ) )^2 + ( Y - y(4) )^2 ) > ( R-r(4) ) ) || (R==Inf) || (r(4)==Inf)
    [x,y,r] = three_kissing_circles;
    [x(4),y(4),r(4),X,Y,R] = descartes( x(1),y(1),r(1), x(2),y(2),r(2), x(3),y(3), r(3) );
end

%Determine a 5th circle within the circumcicle. Ignore the one which is
%circle 3, with positive y
[xx,yy,rr,xxx,yyy,rrr] = descartes( x(1),y(1),r(1), x(2),y(2),r(2), X,Y,-R );
if yy<0
    x(5) = xx; y(5) = yy; r(5) = rr;
else
    x(5) = xxx; y(5) = yyy; r(5) = rrr;
end

%Define a 9x3 array of circle trios which correspond to further kisses
%between current circles. 0 means the X,Y,R circumcircle.
k = [1,2,4; 1,3,4; 2,3,4; 5,1,2; 1,3,0; 3,2,0; 3,1,0; 5,1,0; 5,2,0;];

%%

%Analytic formulae for first few levels of Pappus chain
%https://mathworld.wolfram.com/PappusChain.html. x,y are vectors of circle
%centres, r is a corresponding vector of circle radii and k is an 2*N x 3
%matrix of three circle numbers that could produce a new fourth.
function [x,y,r,k] = pappus(a,N)

%Define starting inner circle radii and centre coordinates
r(1) = a/2; x(1) = r(1); y(1) = 0;
r(2) = (1-a)/2; x(2) = 2*r(1)+r(2); y(2) = 0; r(2) = r(2);

%Principal circles that form a Pappus chain.
j = 2; k=[];
for n=1:N
    j = j+1;
    t = (n^2) * (1-a)^2 + a ;
    x(j) = a*(1+a)/( 2*t);
    y(j) = n*a*(1-a)/t;
    r(j) = (1-a)*a/(2*t);
    k = [k ; j-1, j, 0];  %Vector of new circles kisses
end

%Next level
for n=1:N
    j = j+1;
    t = 4 + 4*n*(n-1)*( (1-a)^2 ) + a*(a-1);
    x(j) = a*(7+a)/( 2*t);
    y(j) = 2*(2*n-1)*a*(1-a)/t;
    r(j) = (1-a)*a/(2*t);
    k = [k ; 1, n+1, n+2]; %Vector of new circles kisses
end

%%

%Defines three element vectors of the centre (x,y) and radius r of three
%(externally) mutually tangent circles. The radii of the first three are randomly chosen.
function [x,y,r] = three_kissing_circles

%Define radii of three kissing circles
r = randi( 7,[1,3] );

%Let circles 1 and 2 kiss along the x axis
x(1) = 0; y(1) = 0; x(2) = r(1)+r(2); y(2) = 0;

%Determine centre of circle 3
x(3) = ( ( r(2) + r(1) )^2 - ( r(3)+r(2) )^2 + ( r(1) + r(3) )^2 )/( 2*( r(2) + r(1) ) );
y(3) = sqrt( ( r(1)+r(3) )^2 - x(3)^2 );

%%

%Determine all possible new circle kisses and generate new circles
function [x,y,r,k] = more_kissing_circles(x,y,r,X,Y,R,k,rmin)

%Step through kiss array and determine new circles, and indeed new possible
%kisses
dim = size(k); new_k = []; j=length(x);
for n=1:dim(1)
    j = j+1;
    if k(n,3)==0
        %Kisses in k(n,:) internally tangent to circumcircle X,Y,R
        [x(j),y(j),r(j),xx,yy,rr] = descartes( x(k(n,1)),y(k(n,1)),r(k(n,1)),...
            x(k(n,2)),y(k(n,2)),r(k(n,2)), X,Y,-R );
        %Choose the circle with the smallest radius
        if rr<r(j)
            x(j) = xx; y(j) = yy; r(j) = rr;
        end
        
        %Check if new circle is larger than minimum radius. If not, don't
        %add circle or work out new kisses.
        if r(j) < rmin
            x(j) = []; y(j) = []; r(j) = []; j = j-1;
        else
            new_k = [ new_k; k(n,1),j,0; k(n,2) , j, 0; j, k(n,1), k(n,2) ];
        end
    else
        %Kiss externally tangent to circles in k(m,:)
        [x(j),y(j),r(j),xx,yy,rr] = descartes( x(k(n,1)),y(k(n,1)),r(k(n,1)),...
            x(k(n,2)),y(k(n,2)),r(k(n,2)),...
            x(k(n,3)),y(k(n,3)),r(k(n,3)) );
        
        %Check if new circle is larger than minimum radius. If not, don't
        %add circle or work out new kisses.
        if r(j) < rmin
            x(j) = []; y(j) = []; r(j) = []; j = j-1;
        else
            new_k = [ new_k; k(n,1),j,k(n,2); k(n,1), j, k(n,3); j, k(n,2), k(n,3) ];
        end
    end
end
k = new_k;

%%

%Determine centre and radius of two circles which kiss three circles
%which are also mutually tangent. If mutual tangency is internal, the radii
%of the third circle input (r3) is negative.
function [x4,y4,r4,x5,y5,r5] = descartes( x1,y1,r1, x2,y2,r2, x3,y3, r3 )

%Convert circle centres into complex numbers z = x+iy
z1 = x1 + 1i*y1; z2 = x2 + 1i*y2; z3 = x3 + 1i*y3;

%Define curvatures of the circles
k1 = 1/r1; k2 = 1/r2; k3 = 1/r3;

%Determine curvature of the fourth and fifth circles and hence their radii
a = k1 + k2 + k3;
b = 2*sqrt( k1*k2 + k2*k3 + k3*k1 );
k4 = a + b; k5 = a - b;
r4 = abs(1/k4); r5 = abs(1/k5);

%Determine complex number centre of fourth circle.
A = z1*k1 + z2*k2 + z3*k3;
B = 2*sqrt( k1*k2*z1*z2 +k2*k3*z2*z3 + k1*k3*z1*z3 );

%Run checks to determine which +/- solution to use.
if r3 > 0
    %Circles 4 and 5 are exernally tangent to circle 1,2,3
    z4 = (A + B)/k4; z5 = (A + B)/k5;
    if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
        z4 = (A - B)/k4;
    end
    
    %Note z5 will be a cicumcircle of z1,z2,z3
    if zcheck_i( z1,z2,z3,z5, r1,r2,r3,r5 )==0
        z5 = (A - B)/k5;
    end
else
    %Circles 1,2,4,5 are internally tangent to circle 3
    z4 = (A + B)/k4; z5 = (A + B)/k5;
    if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
        z4 = (A - B)/k4;
    end
    if zcheck_i( z1,z2,z3,z5, r1,r2,r3,r5 )==0
        z5 = (A - B)/k5;
    end
end

%Determine circle centres
x4 = real(z4); y4 = imag(z4); x5 = real(z5); y5 = imag(z5);

%%

%zcheck_e - checks whether complex descartes formula yields a correct
%solution for the case where z4 is externally tangent to z1,z2,z3
function y = zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )
tol = 1e-12;
check1 = abs( abs(z4-z1) - (r1 +r4) ) < tol;
check2 = abs( abs(z4-z2) - (r2 +r4) ) < tol;
check3 = abs( abs(z4-z3) - (r3 +r4) ) < tol;
y = check1*check2*check3;

%%

%zcheck_i - checks whether complex descartes formula yields a correct
%solution for case where z1,z2,z4 are internally tangent to z3
function y = zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )
tol = 1e-12;

%No overlaps
check1 = abs( abs(z4-z1) - ( r1 + r4 ) ) < tol;
check2 = abs( abs(z4-z2) - ( r2 + r4 ) ) < tol;
check3 = abs( (abs(z3-z4) + r4) < r3) < tol;

%Determine binary flag to see whether all checks are passed
y = check1*check2*check3;

%%

%Plot a circle
function circle( xc,yc,r,faceRGB,lwidth,edge_RGB )
N = ceil( 2000*r ) + 100;
theta = linspace(0,2*pi,N);
x = xc + r*cos(theta); y = yc + r*sin(theta);
patch(x,y,faceRGB,'edgecolor',edge_RGB,'linewidth',lwidth);

%%

%RGB colour arrays for circles
function [colours,Nc] = choose_colormap(c)
if strcmp(c,'random')
    colours = [];
elseif strcmp(c,'tibetan')
    %This is based on Tibetan prayer flags
    colours = [
        36,13,112;
        219,32,20;
        245,226,2;
        255,255,255;
        100, 171,113;
        233,121,16
        ]/255;
elseif strcmp(c,'mondrian')
    %Mondrian
    colours = [
        255,240,1;
        255,1,1;
        1,1,253;
        249,249,249
        ]/255;
else
    %Assume colour string is one of the default MATLAB colormaps
    colours = colormap(c);
end
dim = size(colours); Nc = dim(1);

%End of code