%pappus_chain
% Plots a Pappus Chain construction - a ring of a ring of circles between
% two tangent circles investigated by Pappus of Alexandria in the 3rd
% century AD. https://en.wikipedia.org/wiki/Pappus_chain
% https://mathworld.wolfram.com/PappusChain.html
%
% LAST UPDATED by Andy French July 2023

function pappus_chain

%% INPUTS %%

%Number of circles to plot in principle Pappus chain
N = 50;

%Linewidths and edge colours for circles
lwidth = 0.5; edge_rgb = [0,0,0];

%Define Pappus chain ratio ( 0 < a < 1)
phi = (1+sqrt(5))/2; a = 1/phi; a = 5/7;

%

%Define circle radii that form the arbelos
R1 = a/2;  R2 = (1-a)/2;

%Outer circle
c(1).xc = R1+R2; c(1).yc = 0; c(1).r = R1+R2;

%Inner circles
c(2).xc = R1; c(2).yc = 0; c(2).r = R1;
c(3).xc = 2*R1+R2; c(3).yc = 0; c(3).r = R2;

%Circle counter
j = 3;

%

%% Determine Pappus chain %%

for n=1:N
    %Principle circles - from Mathworld formulae
    j = j+1;
    k = (n^2) * (1-a)^2 + a ;
    c(j).xc = a*(1+a)/( 2*k);
    c(j).yc = n*a*(1-a)/k;
    c(j).r = (1-a)*a/(2*k);
    
    %Additional tangent circles
    j = j+1;
    k = 4 + 4*n*(n-1)*( (1-a)^2 ) + a*(a-1);
    c(j).xc = a*(7+a)/( 2*k);
    c(j).yc = 2*(2*n-1)*a*(1-a)/k;
    c(j).r = (1-a)*a/(2*k);
    
    %Further tangent circles to these... (i)
    j = j+1;
    k = 12 + 3*n*(3*n-4)*( (1-a)^2 ) + a*(4*a-7);
    c(j).xc = a*(17+a)/( 2*k);
    c(j).yc = 3*(3*n-2)*a*(1-a)/k;
    c(j).r = (1-a)*a/(2*k);
    
    %Further tangent circles to these... (ii)
    j = j+1;
    k = 9 + 3*n*(3*n-2)*( (1-a)^2 ) - a*(1-a);
    c(j).xc = a*(17+a)/( 2*k);
    c(j).yc = 3*(3*n-1)*a*(1-a)/k;
    c(j).r = (1-a)*a/(2*k);
    
    %Further tangent circles to these... (iii)
    j = j+1;
    k = 9 + 12*n*(n-1)*( (1-a)^2 ) + a*(4*a-1);
    c(j).xc = a*(17+7*a)/( 2*k);
    c(j).yc = 6*(2*n-1)*a*(1-a)/k;
    c(j).r = (1-a)*a/(2*k);
end

%

%Plot Pappus chain figure
plot_circle( c(1).xc,c(1).yc,c(1).r,edge_rgb,[1,1,1],lwidth  );
for n=2:length(c)
    plot_circle( c(n).xc,c(n).yc,c(n).r,edge_rgb,rand(1,3),lwidth );
    plot_circle( c(n).xc,-c(n).yc,c(n).r,edge_rgb,rand(1,3),lwidth );
end
axis equal; axis tight; grid on; box on; axis off;
title(['Pappus chain : a = ',num2str(a)],'fontsize',14);

%Save figure to a PNG file
filename = ['pappus a=',strrep( num2str(a,3),'.','p'),'.png'];
print( gcf, filename,'-dpng','-r300' );
close(gcf);

%%

%Plot a circle of radius r centered on (x0,y0)
function c = plot_circle( x0,y0,r,edgecolor,facecolor,linewidth )
theta = linspace(0,2*pi,500);
x = x0 + r*cos(theta); y = y0 + r*sin(theta);
c = patch( x,y,facecolor,'edgecolor',edgecolor,'linewidth',linewidth );

%End of code