%Determine centre coordinates and radius of a circle which touches the
% inside of a larger circle, and also two other circles which are inside the
% larger circle.
%
% LAST UPDATED by Andy French July 2023

function three_inner_circles

%Fontsize and linewidths for circles
fsize = 18; lwidth = 1;

%Outer circle centre coordinate and radius
x1 = 0;  y1 = 0; R1 = 10;

%Inner circle radii and elevation angles, such that circles touch inside of outer circle
theta2 = pi/3; R2 = 5;
theta3 = -pi/6; R3 = 3;

%

%Inner circle centre coordinates 
x2 = x1 + (R1-R2)*cos(theta2); y2 = y1 + (R1-R2)*sin(theta2);
x3 = x1 + (R1-R3)*cos(theta3); y3 = y1 + (R1-R3)*sin(theta3);

%Plot outer and inner circles
axes('nextplot','add','fontsize',fsize);
plot_circle( x1,y1,R1,[1,0,0],NaN,lwidth );
plot_circle( x2,y2,R2,[0,1,0],NaN,lwidth );
plot_circle( x3,y3,R3,[0,0,1],NaN,lwidth );
axis equal; axis tight; grid on; box on; grid on; grid minor; 
set(gca,'gridlinestyle','-','minorgridlinestyle','-');
title('Three inner circle intersection')

%Solve for circle which touches two inner and outer circle
[x4,y4,r4,x5,y5,r5] = three_circle_touch( x1,y1,R1,x2,y2,R2,x3,y3,R3 );
plot_circle( x4,y4,r4,[0,0,0],NaN,lwidth );
plot_circle( x5,y5,r5,[0,0,0],NaN,lwidth );

%Check original equations are satisfied. All checks should be eactly zero
zerocheck1 = (R1 - r4)^2 - (x4-x1)^2 - (y4-y1)^2;
zerocheck2 = (R2 + r4)^2 -(x4-x2)^2 - (y4-y2)^2;
zerocheck2 = (R3 + r4)^2 -(x4-x3)^2 - (y4-y3)^2;
zerocheck3 = (R1 - r5)^2 - (x5-x1)^2 - (y5-y1)^2;
zerocheck4 = (R2 + r5)^2 -(x5-x2)^2 - (y5-y2)^2;
zerocheck5 = (R3 + r5)^2 -(x5-x3)^2 - (y5-y3)^2;

%Save figure to a PNG file
print( gcf, 'three inner circles.png','-dpng','-r300' );
close(gcf);

%%

%Solve for centre (x,y) and radius r of circles that touch two circles
%within the larger outer circle with centre (x1,y1) and radius R1
function [xx1,yy1,rr1,xx2,yy2,rr2] = three_circle_touch( x1,y1,R1,x2,y2,R2,x3,y3,R3 )

%Solve assuming larger circle cetre is (0,0)
a = x2 - x1; b = y2 - y1;
c = x3 - x1; d = y3 - y1;

%Compute coefficients of e = A+B*r and f = C+D*r 
%where x = e + x1 and y = f + y1

A = ( R2^2 - R1^2 )*d - (R3^2 - R1^2 )*b + (c^2) * b + (d^2) * b - (a^2) *d - (b^2) *d;
A = A/( 2*c*b - 2*a*d );

B = (R2 + R1)*d - (R3+R1)*b;
B = B/( c*b - a*d );

C = ( R2^2 - R1^2 )*c - (R3^2-R1^2)*a +(c^2)*a +(d^2)*a -(a^2)*c -(b^2)*c;
C = C/(2*d*a - 2*b*c);

D = (R2 + R1)*c - (R3 + R1)*a;
D = D/(d*a - b*c);

%Now solve quadratic for r
aa = 1 - B^2 -D^2;
bb = -2*R1 -2*A*B - 2*C*D;
cc = R1^2 - A^2 - C^2;
rr1 = ( - bb + sqrt( bb^2 -4*aa*cc ) )/(2*aa);
rr2 = ( - bb - sqrt( bb^2 -4*aa*cc ) )/(2*aa);

%Determine circle centre
xx1 = A + B*rr1 + x1; yy1 = C + D*rr1 + y1;
xx2 = A + B*rr2 + x1; yy2 = C + D*rr2 + y1;

%%

%Plot a circle of radius r centered on (x0,y0)
function c = plot_circle( x0,y0,r,edgecolor,facecolor,linewidth )
N = 1000; %Number of evaluation points of circle
theta = linspace(0,2*pi,N);
x = x0 + r*cos(theta); 
y = y0 + r*sin(theta);
c = patch( x,y,facecolor,'edgecolor',edgecolor,'linewidth',linewidth );

%End of code
