%apollonian_gasket
% Generates the Apollonian Gasket fractal. Plots a Pappus Chain
% construction - a ring of a ring of circles between two tangent circles
% investigated by Pappus of Alexandria in the 3rd century AD.
%
% https://en.wikipedia.org/wiki/Pappus_chain
% https://mathworld.wolfram.com/PappusChain.html
% https://en.wikipedia.org/wiki/Descartes%27_theorem
%
% Extra tangent circles are then added, using Descartes' formula,
% approximating an Apollonian Gasket fractal.
%
% LAST UPDATED by Andy French July 2023

function apollonian_gasket

%% INPUTS %%

%Define Pappus chain ratio ( 0 < a < 1)
phi = (1+sqrt(5))/2; a = 1/phi;

%Number of circles to plot in principle Pappus chain
N = 50;

%Max number of iterations using Descartes' formula
M = 1e5;

%Max number of circles to plot
C = 1000;

%Linewidths and edge colours for circles
lwidth = 0.3; edge_rgb = [0,0,0];

%Minimum circle radius
minr = 1e-4;

%Fixed colour pattern (blank means random RGB colours) 

%This is based on Tibetan prayer flags
colours = [ 
    36,13,112;
    219,32,20;
    245,226,2;
    255,255,255;
    100, 171,113;
    233,121,16
    ]/255;

%Mondrian
colours = [
    255,240,1;
    255,1,1;
    1,1,253;
    249,249,249
    ]/255;

%

%Display starting message and start clock
clc; disp(''); disp(' DESCARTES KISSING CIRCLES GENERATING THE APOLLONIAN GASKET ')
t0 = tic;

%Outer circumscribing circle
X = 1/2; Y = 0; R = 1/2;

%Determine coordinates of principle Pappus chain (and inner chain) using
%analytical formulae
[x,y,r,k] = pappus(a,N);

%Step through kisses array and work out new circles. Update the kisses
%array accordingly, and remove kisses once used.
n = length(x); j = 1;
while (j < M) 
    dim = size(k); new_k = [];
    for m=1:dim(1)
        j = j+1;
        if (j >= M) 
            break
        elseif k(m,3)==0
            %Third kiss is the circumcircle
            [xx,yy,rr] = descartes_i( x(k(m,1)),y(k(m,1)),r(k(m,1)),...
                x(k(m,2)),y(k(m,2)),r(k(m,2)),X,Y,R );
            if (rr > minr)
                %Only add to circles (and kiss vector) if new circle has a
                %greater radius than minr
                n = n+1;
                x(n) = xx; y(n) = yy; r(n) = rr;
                new_k = [ new_k; n,k(m,1),0 ; n,k(m,2),0; n,k(m,1),k(m,2) ];           
            end
        else
            %Three circles kissing externally
            [xx,yy,rr,xxx,yyy,zzz] = descartes_e( x(k(m,1)),y(k(m,1)),r(k(m,1)),...
                x(k(m,2)),y(k(m,2)),r(k(m,2)),...
                x(k(m,3)),y(k(m,3)),r(k(m,3)) );
            if (rr > minr)
                %Only add to circles (and kiss vector) if new circle has a
                %greater radius than minr
                n = n+1; 
                x(n) = xx; y(n) = yy; r(n) = rr;
                new_k = [ new_k; n,k(m,1),k(m,2) ; n,k(m,1),k(m,3); n,k(m,2),k(m,3) ];
            end
        end      
    end
    k = new_k; %Update kiss matrix, removing those that have now been used to generate new circles
    if isempty(k); break; end
end

%Sort circles by radius
[r,i] = sort(r, 'descend'); x = x(i); y =  y(i);

%Display message to command window
disp( [' ... Gasket calculated in ',num2str(toc(t0)),'s.'] );
disp( ' ... Now plotting - this may take a while!' )

%

%Set up a blank figure window with a blank axes that can be added to with
%plot commands
fig = figure('renderer','opengl',...
    'units','normalized','position',[0.1,0.1,0.8,0.8],...
    'name','... Constructing Apollonius Gasket'); axes('NextPlot','add'); 

%Plot circles
circle( X,Y,R,[1,1,1] ); t = title('... Plotting Apollonius Gasket - please wait','fontsize',18);
axis equal; axis manual; axis off; j=1; c = 1; dim = size(colours);
for n=1:C
    %Flush graphics buffer every 100 circles. This will yield an animated
    %effect as the small circles are added.
    if j==10; drawnow; j=1; end
    if isempty(colours)
        RGB = rand(1,3);
    else
        RGB = colours(c,:);
    end
    c = c + 1; if c>dim(1); c = 1; end
    circle( x(n),y(n),r(n),RGB );
    circle( x(n),-y(n),r(n),RGB );
    j = j + 1;
end

%Print figure and close
drawnow; delete(t);
print( gcf,['Apollonian gasket a=',strrep(num2str(a,3),'.','-'),'.png'],'-dpng','-r600' ); close(gcf);
disp( [' ... Gasket printed to a PNG file in ',num2str(toc(t0)),'s.']);

%%

%Determine centre and radius of a fourth circle which kisses three circles
%which also kiss externally. Also determine centre and radius of a circle
%which circumscribes the original three.
function [x4,y4,r4,xc,yc,rc] = descartes_e( x1,y1,r1, x2,y2,r2, x3,y3, r3 )

%Convert circle centres into complex numbers z = x+iy
z1 = x1 + 1i*y1; z2 = x2 + 1i*y2; z3 = x3 + 1i*y3;

%Define curvatures of the circles
k1 = 1/r1; k2 = 1/r2; k3 = 1/r3;

%Determine curvature of the fourth circle
a = k1 + k2 + k3;
b = 2*sqrt( k1*k2 + k2*k3 + k3*k1 );
k = a + b;

%Determine complex number centre of fourth circle.
A = z1*k1 + z2*k2 + z3*k3;
B = 2*sqrt( k1*k2*z1*z2 +k2*k3*z2*z3 + k1*k3*z1*z3 );

%Run checks to determine which +/- solution to use.
z4 = (A + B)/k; r4 = abs(1/k);
if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
    z4 = (A - B)/k;
    if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
        k = a - b; r4 = abs(1/k); z4 = (A - B)/k;
        if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
            z4 = (A + B)/k;
            if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
                z4 = NaN; r4 = NaN;
            end
        end
    end
end
x4 = real(z4); y4 = imag(z4);

%Determine curvature of the circumcircle
k = a - b;

%Determine complex number centre of fourth circle.
zc = (A - B)/k;
xc = real(zc); yc = imag(zc); rc = abs(1/k);

%%

%Determine centre and radius of a circle which kisses two circles
%which also kiss externally, and such that all circles kiss the inside
%of a circle with centre x3,y3 and radius r3.
function [x4,y4,r4] = descartes_i( x1,y1,r1, x2,y2,r2, x3,y3, r3 )

%Convert circle centres into complex numbers z = x+iy
z1 = x1 + 1i*y1; z2 = x2 + 1i*y2; z3 = x3 + 1i*y3;

%Define curvatures of the circles. Note circle 3 is internally tangent to the other circles.
k1 = 1/r1; k2 = 1/r2; k3 = -1/r3;

%Determine curvature of the fourth circle
a = k1 + k2 + k3;
b = 2*sqrt( k1*k2 + k2*k3 + k3*k1 );
k = a + b;

%Determine complex number centre of fourth circle.
A = z1*k1 + z2*k2 + z3*k3;
B = 2*sqrt( k1*k2*z1*z2 +k2*k3*z2*z3 + k1*k3*z1*z3 );

%Run checks to determine which +/- solution to use.
z4 = (A + B)/k; r4 = abs(1/k);
if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
    z4 = (A - B)/k;
    if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
        k = a - b; r4 = abs(1/k); z4 = (A - B)/k;
        if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
            z4 = (A + B)/k;
            if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
                z4 = NaN; r4 = NaN;
            end
        end
    end
end
x4 = real(z4); y4 = imag(z4);

%%

%zchecki - checks whether complex descartes formula yields a correct
%solution for the external case
function y = zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )
tol = 1e-12;
check1 = abs( abs(z4-z1) - (r1 +r4) ) < tol;
check2 = abs( abs(z4-z2) - (r2 +r4) ) < tol;
check3 = abs( abs(z4-z3) - (r3 +r4) ) < tol;
y = check1*check2*check3;

%%

%zchecko - checks whether complex descartes formula yields a correct
%solution for the inside case
function y = zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )
tol = 1e-12;

%No overlaps
check1 = abs( abs(z4-z1) - ( r1 + r4 ) ) < tol;
check2 = abs( abs(z4-z2) - ( r2 + r4 ) ) < tol;
check3 = abs( abs(z4-z3) - ( r3 - r4 ) ) < tol;

%Determine binary flag to see whether all checks are passed
y = check1*check2*check3;

%%

%Plot a circle
function circle( xc,yc,r,faceRGB )
N = ceil( 2000*r ) + 100;
theta = linspace(0,2*pi,N);
x = xc + r*cos(theta); y = yc + r*sin(theta);
patch(x,y,faceRGB,'edgecolor',[0,0,0],'linewidth',0.5);

%%

%Plot a circle, or indeed a vector of circles
function circlevec( xc,yc,r,faceRGB )
N = 500; theta = linspace(0,2*pi,N).'; C = ones(1,length(xc),3);
theta = repmat( theta, [1,length(xc)] );
xc = repmat( xc, [N,1] ); yc = repmat( yc, [N,1] ); r = repmat( r, [N,1] );
x = xc + r.*cos(theta); y = yc + r.*sin(theta);
C(1,:,:) = faceRGB(:,:); patch(x,y,C,'edgecolor',[0,0,0],'linewidth',0.5);

%%

%Analytic formulae for first few levels of Pappus chain
%https://mathworld.wolfram.com/PappusChain.html. x,y are vectors of circle
%centres, r is a corresponding vector of circle radii and k is an 2*N x 3
%matrix of three circle numbers that could produce a new fourth.
function [x,y,r,k] = pappus(a,N)

%Define starting inner circle radii and centre coordinates
r(1) = a/2; x(1) = r(1); y(1) = 0;
r(2) = (1-a)/2; x(2) = 2*r(1)+r(2); y(2) = 0; r(2) = r(2);

%Principal circles that form a Pappus chain.
j = 2; k=[];
for n=1:N
    j = j+1;
    t = (n^2) * (1-a)^2 + a ;
    x(j) = a*(1+a)/( 2*t);
    y(j) = n*a*(1-a)/t;
    r(j) = (1-a)*a/(2*t);
    k = [k ; j-1, j, 0];  %Vector of new circles kisses
end

%Next level
for n=1:N
    j = j+1;
    t = 4 + 4*n*(n-1)*( (1-a)^2 ) + a*(a-1);
    x(j) = a*(7+a)/( 2*t);
    y(j) = 2*(2*n-1)*a*(1-a)/t;
    r(j) = (1-a)*a/(2*t);
    k = [k ; 1, n+1, n+2]; %Vector of new circles kisses
end

%End of code