%apollonian_gasket_asymmetric
% Generates the Apollonian Gasket fractal, starting with three mutually
% tangent circles.
%
% https://en.wikipedia.org/wiki/Pappus_chain
% https://mathworld.wolfram.com/PappusChain.html
% https://en.wikipedia.org/wiki/Descartes%27_theorem
%
% LAST UPDATED by Andy French July 2023

function apollonian_gasket_asymmetric

%% INPUTS %%

%Define radii of three kissing circles
r = [5,2,3];

%Max number of circles to find
M = 5e4;

%Max number of circles to plot
C = 1000;

%Linewidths and edge colours for circles
lwidth = 0.3; edge_rgb = [0,0,0];

%Minimum circle radius
minr = 1e-4;

%Fixed colour pattern (blank means random RGB colours)

%This is based on Tibetan prayer flags
colours = [
    36,13,112;
    219,32,20;
    245,226,2;
    255,255,255;
    100, 171,113;
    233,121,16
    ]/255;

%Mondrian
colours = [
    255,240,1;
    255,1,1;
    1,1,253;
    249,249,249
    ]/255;

%

%Display starting message and start clock
clc; disp(''); disp(' DESCARTES KISSING CIRCLES GENERATING THE APOLLONIAN GASKET ')
t0 = tic;

%%

%Let circles 1 and 2 kiss along the x axis
x(1) = 0; y(1) = 0; x(2) = r(1)+r(2); y(2) = 0;

%Determine centre of circle 3
x(3) = ( (r(2) + r(1) )^2 - (r(3)+r(2))^2 + (r(1) + r(3))^2 )/( 2*(r(2) + r(1)) );
y(3) = sqrt( ( r(1)+r(3) )^2 - x(3)^2 );

%Determine two more circles that kiss the three circles generated so far.
%X,Y,R describes the outer circumcircle circle bounding all circles.
[x(4),y(4),r(4),X,Y,R] = three_outer_circle_touch( x(1),y(1),r(1),x(2),y(2),r(2),x(3),y(3),r(3) );

%And two more circle that touches the circumcircle
[x(5),y(5),r(5),x(6),y(6),r(6)] = three_inner_circle_touch( x(1),y(1),r(1),x(3),y(3),r(3),X,Y,R );

%Initialize kisses array. 0 corresponds to the circumcircle.
k = [ 1,3,0; 2,3,0; 1,2,0; 1,3,4; 1,3,3; 1,2,4; 5,1,0; 5,3,0; 5,1,3; 6,1,0; 6,3,0; 6,1,3 ];

%Set up a blank figure window with a blank axes that can be added to with
%plot commands
fig = figure('renderer','opengl',...
    'units','normalized','position',[0.1,0.1,0.8,0.8],...
    'name','... Constructing Apollonius Gasket'); axes('NextPlot','add');

%Plot initial setup
circle( X,Y,R,[1,1,1] ); t = title('... Calculating Apollonius Gasket - please wait','fontsize',18);
axis equal; axis manual; axis off;
for n=1:length(x)
    circle( x(n),y(n),r(n),rand(1,3) );
end
drawnow;

%Step through kisses array and work out new circles. Update the kisses
%array accordingly, and remove kisses once used.
n = length(x); j = 1;
while (j < M)
    dim = size(k); new_k = [];
    for m=1:dim(1)
        j = j+1;
        if (j >= M)
            break
        elseif k(m,3)==0
            %Third kiss is the circumcircle
            [xx,yy,rr] = descartes_i( x(k(m,1)),y(k(m,1)),r(k(m,1)),...
                x(k(m,2)),y(k(m,2)),r(k(m,2)),X,Y,R  );
            if (rr > minr)
                %Only add to circles (and kiss vector) if new circle has a
                %greater radius than minr
                n = n+1;
                r(n)=rr; x(n) = xx; y(n) = yy;
                new_k = [ new_k; n,k(m,1),0 ; n,k(m,2),0; n,k(m,1),k(m,2) ];
            end
        else
            %Three circles kissing externally
            [xx,yy,rr,xxx,yyy,rrr] = descartes_e( x(k(m,1)),y(k(m,1)),r(k(m,1)),...
                x(k(m,2)),y(k(m,2)),r(k(m,2)),...
                x(k(m,3)),y(k(m,3)),r(k(m,3))  );
            
            if (rr > minr)
                %Only add to circles (and kiss vector) if new circle has a
                %greater radius than minr
                n = n+1;
                r(n)=rr; x(n) = xx; y(n) = yy;
                new_k = [ new_k; n,k(m,1),k(m,2) ; n,k(m,1),k(m,3); n,k(m,2),k(m,3) ];
            end
        end
    end
    k = new_k; %Update kiss matrix, removing those that have now been used to generate new circles
    if isempty(k); break; end
end

%Sort circles by radius
[r,i] = sort(r, 'descend'); x = x(i); y =  y(i);

%Display message to command window
disp( [' ... Gasket calculated in ',num2str(toc(t0)),'s.'] );
disp( ' ... Now plotting - this may take a while!' )

%

%Plot circles
clf;
circle( X,Y,R,[1,1,1] ); t = title('... Plotting Apollonius Gasket - please wait','fontsize',18);
axis equal; axis manual; axis off; j=1; c = 1; dim = size(colours);
for n=1:C
    %Flush graphics buffer every 100 circles. This will yield an animated
    %effect as the small circles are added.
    if j==10; drawnow; j=1; end
    if isempty(colours)
        RGB = rand(1,3);
    else
        RGB = colours(c,:);
    end
    c = c + 1; if c>dim(1); c = 1; end
    circle( x(n),y(n),r(n),RGB );
    j = j + 1;
end

%Print figure and close
t = ['r1=',num2str(r(1)),', r2=',num2str(r(2)),', r3=',num2str(r(3))];
drawnow; title(t,'fontsize',14);
print( gcf,['Apollonian gasket a=',strrep(t,'.','-'),'.png'],'-dpng','-r600' ); close(gcf);
disp( [' ... Gasket printed to a PNG file in ',num2str(toc(t0)),'s.']);

%%

%Plot a circle
function circle( xc,yc,r,faceRGB )
N = ceil( 2000*r ) + 100;
theta = linspace(0,2*pi,N);
x = xc + r*cos(theta); y = yc + r*sin(theta);
patch(x,y,faceRGB,'edgecolor',[0,0,0],'linewidth',0.5);

%%

%Solve for centre (x,y) and radius r of circle that touches three other circles
% Code from https://math.stackexchange.com/questions/3100828/calculate-the-circle-that-touches-three-other-circles
function [x1,y1,r1,x2,y2,r2] = three_outer_circle_touch( x_1,y_1,r_1,x_2,y_2,r_2,x_3,y_3,r_3 )

% Find constant of circle #2- circle #1
K_a = -r_1^2+r_2^2+x_1^2-x_2^2+y_1^2-y_2^2;

% Find constant of circle #3- cirlce #1
K_b = -r_1^2+r_3^2+x_1^2-x_3^2+y_1^2-y_3^2;

% Find constants of [x=A_0+A_1*r, y=B_0+B_1*r]
D = x_1*(y_2-y_3)+x_2*(y_3-y_1)+x_3*(y_1-y_2);
A_0 =(K_a*(y_1-y_3)+K_b*(y_2-y_1))/(2*D);
B_0 = -(K_a*(x_1-x_3)+K_b*(x_2-x_1))/(2*D);
A_1 = -(r_1*(y_2-y_3)+r_2*(y_3-y_1)+r_3*(y_1-y_2))/D;
B_1 = (r_1*(x_2-x_3)+r_2*(x_3-x_1)+r_3*(x_1-x_2))/D;

% Find constants of C_0 + 2*C_1*r + C_2^2 = 0
C_0 = A_0^2-2*A_0*x_1+B_0^2-2*B_0*y_1-r_1^2+x_1^2+y_1^2;
C_1 = A_0*A_1-A_1*x_1+B_0*B_1-B_1*y_1-r_1;
C_2 = A_1^2+B_1^2-1;

% Solve for r,x,y
r1=( -sqrt(C_1^2-C_0*C_2)-C_1)/C_2;
x1 = A_0 + A_1*r1;
y1 = B_0 + B_1*r1;
r2=( sqrt(C_1^2-C_0*C_2)-C_1)/C_2;
x2 = A_0 + A_1*r2;
y2 = B_0 + B_1*r2;
r2 = abs(r2); %Circumcircle has negative curvature

%%

%Solve for centre (x,y) and radius r of circles that touch two circles
%within the larger outer circle with centre (x1,y1) and radius R1
function [xx1,yy1,rr1,xx2,yy2,rr2] = three_inner_circle_touch( x2,y2,R2,x3,y3,R3,x1,y1,R1 )

%Solve assuming larger circle cetre is (0,0)
a = x2 - x1; b = y2 - y1;
c = x3 - x1; d = y3 - y1;

%Compute coefficients of e = A+B*r and f = C+D*r
%where x = e + x1 and y = f + y1

A = ( R2^2 - R1^2 )*d - (R3^2 - R1^2 )*b + (c^2) * b + (d^2) * b - (a^2) *d - (b^2) *d;
A = A/( 2*c*b - 2*a*d );

B = (R2 + R1)*d - (R3+R1)*b;
B = B/( c*b - a*d );

C = ( R2^2 - R1^2 )*c - (R3^2-R1^2)*a +(c^2)*a +(d^2)*a -(a^2)*c -(b^2)*c;
C = C/(2*d*a - 2*b*c);

D = (R2 + R1)*c - (R3 + R1)*a;
D = D/(d*a - b*c);

%Now solve quadratic for r
aa = 1 - B^2 -D^2;
bb = -2*R1 -2*A*B - 2*C*D;
cc = R1^2 - A^2 - C^2;
rr1 = ( - bb + sqrt( bb^2 -4*aa*cc ) )/(2*aa);
rr2 = ( - bb - sqrt( bb^2 -4*aa*cc ) )/(2*aa);

%Determine circle centre
xx1 = A + B*rr1 + x1; yy1 = C + D*rr1 + y1;
xx2 = A + B*rr2 + x1; yy2 = C + D*rr2 + y1;

%Make sure radii are positive
rr1 = abs(rr1);
rr2 = abs(rr2);

%%

%Determine centre and radius of a fourth circle which kisses three circles
%which also kiss externally. Also determine centre and radius of a circle
%which circumscribes the original three.
function [x4,y4,r4,xc,yc,rc] = descartes_e( x1,y1,r1, x2,y2,r2, x3,y3, r3 )

%Convert circle centres into complex numbers z = x+iy
z1 = x1 + 1i*y1; z2 = x2 + 1i*y2; z3 = x3 + 1i*y3;

%Define curvatures of the circles
k1 = 1/r1; k2 = 1/r2; k3 = 1/r3;

%Determine curvature of the fourth circle
a = k1 + k2 + k3;
b = 2*sqrt( k1*k2 + k2*k3 + k3*k1 );
k = a + b;

%Determine complex number centre of fourth circle.
A = z1*k1 + z2*k2 + z3*k3;
B = 2*sqrt( k1*k2*z1*z2 +k2*k3*z2*z3 + k1*k3*z1*z3 );

%Run checks to determine which +/- solution to use.
z4 = (A + B)/k; r4 = abs(1/k);
if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
    z4 = (A - B)/k;
    if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
        k = a - b; r4 = abs(1/k); z4 = (A - B)/k;
        if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
            z4 = (A + B)/k;
            if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
                z4 = NaN; r4 = NaN;
            end
        end
    end
end
x4 = real(z4); y4 = imag(z4);

%Determine curvature of the circumcircle
k = a - b;

%Determine complex number centre of fourth circle.
zc = (A - B)/k;
xc = real(zc); yc = imag(zc); rc = abs(1/k);

%%

%Determine centre and radius of a circle which kisses two circles
%which also kiss externally, and such that all circles kiss the inside
%of a circle with centre x3,y3 and radius r3.
function [x4,y4,r4] = descartes_i( x1,y1,r1, x2,y2,r2, x3,y3, r3 )

%Convert circle centres into complex numbers z = x+iy
z1 = x1 + 1i*y1; z2 = x2 + 1i*y2; z3 = x3 + 1i*y3;

%Define curvatures of the circles. Note circle 3 is internally tangent to the other circles.
k1 = 1/r1; k2 = 1/r2; k3 = -1/r3;

%Determine curvature of the fourth circle
a = k1 + k2 + k3;
b = 2*sqrt( k1*k2 + k2*k3 + k3*k1 );
k = a + b;

%Determine complex number centre of fourth circle.
A = z1*k1 + z2*k2 + z3*k3;
B = 2*sqrt( k1*k2*z1*z2 +k2*k3*z2*z3 + k1*k3*z1*z3 );

%Run checks to determine which +/- solution to use.
z4 = (A + B)/k; r4 = abs(1/k);
if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
    z4 = (A - B)/k;
    if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
        k = a - b; r4 = abs(1/k); z4 = (A - B)/k;
        if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
            z4 = (A + B)/k;
            if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
                z4 = NaN; r4 = NaN;
            end
        end
    end
end
x4 = real(z4); y4 = imag(z4);

%%

%zchecki - checks whether complex descartes formula yields a correct
%solution for the external case
function y = zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )
tol = 1e-12;
check1 = abs( abs(z4-z1) - (r1 +r4) ) < tol;
check2 = abs( abs(z4-z2) - (r2 +r4) ) < tol;
check3 = abs( abs(z4-z3) - (r3 +r4) ) < tol;
y = check1*check2*check3;

%%

%zchecko - checks whether complex descartes formula yields a correct
%solution for the inside case
function y = zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )
tol = 1e-12;

%No overlaps
check1 = abs( abs(z4-z1) - ( r1 + r4 ) ) < tol;
check2 = abs( abs(z4-z2) - ( r2 + r4 ) ) < tol;
check3 = abs( abs(z4-z3) - ( r3 - r4 ) ) < tol;

%Determine binary flag to see whether all checks are passed
y = check1*check2*check3;

%End of code