% descartes_kissing_circles
% "In geometry, Descartes' theorem states that
% for every four kissing, or mutually tangent, circles, the radii of the
% circles satisfy a certain quadratic equation. By solving this equation,
% one can construct a fourth circle tangent to three given, mutually
% tangent circles. The theorem is named after Ren Descartes, who stated it
% in 1643." https://en.wikipedia.org/wiki/Descartes%27_theorem
%
% LAST UPDATED by Andy French July 2023

function descartes_kissing_circles

%Define radii of three kissing circles
r = [1,1,3];

%Max number of extra circles to find in construction of the Apollonian Gasket
nmax = 1000;

%Define solution method. If not 'descartes' then use algebraic solution for
%detrmining the fourth circle.
method = 'descartes';
method = 'algebraic';

%%

%Display starting message and start clock
clc; disp(''); disp(' DESCARTES KISSING CIRCLES GENERATING THE APOLLONIAN GASKET ')
t0 = tic;

%Set up a blank figure window with a blank axes that can be added to with
%plot commands
figure('renderer','opengl'); axes('NextPlot','add'); axis equal; axis off;

%Let circles 1 and 2 kiss along the x axis
x(1) = 0; y(1) = 0; x(2) = r(1)+r(2); y(2) = 0;

%Determine centre of circle 3
x(3) = ( (r(2) + r(1) )^2 - (r(3)+r(2))^2 + (r(1) + r(3))^2 )/( 2*(r(2) + r(1)) );
y(3) = sqrt( ( r(1)+r(3) )^2 - x(3)^2 );

%Determine centre and radius of circle 4, which should kiss the other
%three, and a circumcircle of circles 1,2,3, of centre (X,Y) and radius R
[x(4),y(4),r(4),X,Y,R] = descartes_e( x(1),y(1),r(1), x(2),y(2),r(2), x(3),y(3), r(3) );

%Define an array of kisses so far. Zero means the outer circle, other numbers
%correspond to circles
k = [ 1,2,0 ; 1,3,0 ; 2, 3, 0 ; 4,1,2 ; 4,1,3; 4,2,3];

%Step through kisses array and work out new circles. Update the kisses
%array accordingly, and remove kisses once used
n=4;
while n<nmax
    dim = size(k); new_k = [];
    for m=1:dim(1)
        n = n+1;
        if k(m,3)==0
            %Third kiss is the circumcircle
            if strcmp( method, 'descartes')
                [x(n),y(n),r(n)] = descartes_i( x(k(m,1)),y(k(m,1)),r(k(m,1)),...
                    x(k(m,2)),y(k(m,2)),r(k(m,2)),X,Y,R );
            else
                [x(n),y(n),r(n)] = three_inner_circle_touch( X,Y,R, x(k(m,1)),y(k(m,1)),r(k(m,1)),...
                    x(k(m,2)),y(k(m,2)),r(k(m,2)) );
            end
        else
            %Three circles kissing externally
            if strcmp( method, 'descartes')
                [x(n),y(n),r(n),xx,yy,zz] = descartes_e( x(k(m,1)),y(k(m,1)),r(k(m,1)),...
                    x(k(m,2)),y(k(m,2)),r(k(m,2)),...
                    x(k(m,3)),y(k(m,3)),r(k(m,3)) );
            else
                [x(n),y(n),r(n)] = three_circle_touch( x(k(m,1)),y(k(m,1)),r(k(m,1)),...
                    x(k(m,2)),y(k(m,2)),r(k(m,2)),...
                    x(k(m,3)),y(k(m,3)),r(k(m,3)) );
            end
        end
        new_k = [ new_k; n,k(m,1),k(m,2) ; n,k(m,1),k(m,3); n,k(m,2),k(m,3) ];
    end
    k = new_k;
end

%Plot circles
circle( X,Y,R,[1,1,1] );
circle( x,y,r,rand(length(x),3) );

%Print figure and close
drawnow;
print( gcf,'Descartes kissing circles.png','-dpng','-r300' ); close(gcf);
disp( [' ... Gasket calculated in ',num2str(toc(t0)),'s using ',method,' method.']);

%%

%Determine centre and radius of a fourth circle which kisses three circles
%which also kiss externally. Also determine centre and radius of a circle
%which circumscribes the original three.
function [x4,y4,r4,xc,yc,rc] = descartes_e( x1,y1,r1, x2,y2,r2, x3,y3, r3 )

%Convert circle centres into complex numbers z = x+iy
z1 = x1 + 1i*y1; z2 = x2 + 1i*y2; z3 = x3 + 1i*y3;

%Define curvatures of the circles
k1 = 1/r1; k2 = 1/r2; k3 = 1/r3;

%Determine curvature of the fourth circle
a = k1 + k2 + k3;
b = 2*sqrt( k1*k2 + k2*k3 + k3*k1 );
k = a + b;

%Determine complex number centre of fourth circle.
A = z1*k1 + z2*k2 + z3*k3;
B = 2*sqrt( k1*k2*z1*z2 +k2*k3*z2*z3 + k1*k3*z1*z3 );

%Run checks to determine which +/- solution to use.
z4 = (A + B)/k; r4 = abs(1/k);
if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
    z4 = (A - B)/k;
    if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
        k = a - b; r4 = abs(1/k); z4 = (A - B)/k;
        if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
            z4 = (A + B)/k;
            if zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )==0
                z4 = NaN; r4 = NaN;
            end
        end
    end
end
x4 = real(z4); y4 = imag(z4);

%Determine curvature of the circumcircle
k = a - b;

%Determine complex number centre of fourth circle.
zc = (A - B)/k;
xc = real(zc); yc = imag(zc); rc = abs(1/k);

%%

%Determine centre and radius of a circle which kisses two circles
%which also kiss externally, and such that all circles kiss the inside
%of a circle with centre x3,y3 and radius r3.
function [x4,y4,r4] = descartes_i( x1,y1,r1, x2,y2,r2, x3,y3, r3 )

%Convert circle centres into complex numbers z = x+iy
z1 = x1 + 1i*y1; z2 = x2 + 1i*y2; z3 = x3 + 1i*y3;

%Define curvatures of the circles. Note circle 3 is internally tangent to the other circles.
k1 = 1/r1; k2 = 1/r2; k3 = -1/r3;

%Determine curvature of the fourth circle
a = k1 + k2 + k3;
b = 2*sqrt( k1*k2 + k2*k3 + k3*k1 );
k = a + b;

%Determine complex number centre of fourth circle.
A = z1*k1 + z2*k2 + z3*k3;
B = 2*sqrt( k1*k2*z1*z2 +k2*k3*z2*z3 + k1*k3*z1*z3 );

%Run checks to determine which +/- solution to use.
z4 = (A + B)/k; r4 = abs(1/k);
if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
    z4 = (A - B)/k;
    if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
        k = a - b; r4 = abs(1/k); z4 = (A - B)/k;
        if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
            z4 = (A + B)/k;
            if zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )==0
                z4 = NaN; r4 = NaN;
            end
        end
    end
end
x4 = real(z4); y4 = imag(z4);

%%

%zchecki - checks whether complex descartes formula yields a correct
%solution for the external case
function y = zcheck_e( z1,z2,z3,z4, r1,r2,r3,r4 )
tol = 1e-12;
check1 = abs( abs(z4-z1) - (r1 +r4) ) < tol;
check2 = abs( abs(z4-z2) - (r2 +r4) ) < tol;
check3 = abs( abs(z4-z3) - (r3 +r4) ) < tol;
y = check1*check2*check3;

%%

%zchecko - checks whether complex descartes formula yields a correct
%solution for the inside case
function y = zcheck_i( z1,z2,z3,z4, r1,r2,r3,r4 )
tol = 1e-12;

%No overlaps
check1 = abs( abs(z4-z1) - ( r1 + r4 ) ) < tol;
check2 = abs( abs(z4-z2) - ( r2 + r4 ) ) < tol;
check3 = abs( abs(z4-z3) - ( r3 - r4 ) ) < tol;

%Determine binary flag to see whether all checks are passed
y = check1*check2*check3;

%%

%Plot a circle, or indeed a vector of circles
function circle( xc,yc,r,faceRGB )
N = 500; theta = linspace(0,2*pi,N).'; C = ones(1,length(xc),3);
theta = repmat( theta, [1,length(xc)] );
xc = repmat( xc, [N,1] ); yc = repmat( yc, [N,1] ); r = repmat( r, [N,1] );
x = xc + r.*cos(theta); y = yc + r.*sin(theta);
C(1,:,:) = faceRGB(:,:); patch(x,y,C,'edgecolor',[0,0,0],'linewidth',0.5);

%%

%Solve for centre (x,y) and radius r of circle that touches three other circles
% Code from https://math.stackexchange.com/questions/3100828/calculate-the-circle-that-touches-three-other-circles
function [x,y,r] = three_circle_touch( x1,y1,r1,x2,y2,r2,x3,y3,r3 )

% Find constant of circle #2- circle #1
Ka = -r1^2 + r2^2 + x1^2 - x2^2 + y1^2 - y2^2;

% Find constant of circle #3- circle #1
Kb = -r1^2 + r3^2 + x1^2 - x3^2 + y1^2 - y3^2;

% Find constants of [x=A0+A1*r, y=B0+B1*r]
D = x1*( y2 - y3 ) + x2*( y3-y1 ) + x3*( y1-y2 );
A0 =( Ka*( y1-y3 )+Kb*( y2-y1 ) )/(2*D);
B0 = -( Ka*( x1-x3 ) + Kb*( x2-x1 ))/(2*D);
A1 = -( r1*( y2-y3 ) + r2*( y3-y1 ) + r3*( y1-y2 ) )/D;
B1 = ( r1*( x2-x3 ) + r2*( x3-x1 ) + r3*( x1-x2 ) )/D;

% Find constants of C0 + 2*C1*r + C2^2 = 0
C0 = A0^2 -2*A0*x1 + B0^2 - 2*B0*y1 -r1^2 + x1^2 + y1^2;
C1 = A0*A1 -A1*x1 + B0*B1 -B1*y1-r1;
C2 = A1^2 + B1^2-1;

% Solve for r,x,y
r=( -sqrt( C1^2-C0*C2 ) -C1 )/C2;
x = A0 + A1*r;
y = B0 + B1*r;

%%

%Solve for centre (x,y) and radius r of circle that touches two circles
%within the larger outer circle with centre (x1,y1) and radius R1
function [x,y,r] = three_inner_circle_touch( x1,y1,r1,x2,y2,r2,x3,y3,r3 )

%Solve assuming larger circle cetre is (0,0)
a = x2 - x1; b = y2 - y1;
c = x3 - x1; d = y3 - y1;

%Compute coefficients of e = A+B*r and f = C+D*r
%where x = e + x1 and y = f + y1

A = ( r2^2 - r1^2 )*d - (r3^2 - r1^2 )*b + (c^2) * b + (d^2) * b - (a^2) *d - (b^2) *d;
A = A/( 2*c*b - 2*a*d );
B = (r2 + r1)*d - (r3+r1)*b;
B = B/( c*b - a*d );
C = ( r2^2 - r1^2 )*c - (r3^2-r1^2)*a +(c^2)*a +(d^2)*a -(a^2)*c -(b^2)*c;
C = C/(2*d*a - 2*b*c);
D = (r2 + r1)*c - (r3 + r1)*a;
D = D/(d*a - b*c);

%Now solve quadratic for r
aa = 1 - B^2 -D^2;
bb = -2*r1 -2*A*B - 2*C*D;
cc = r1^2 - A^2 - C^2;

%Take smallest r solution
rplus = ( - bb + sqrt( bb^2 -4*aa*cc ) )/(2*aa);
rminus = ( - bb - sqrt( bb^2 -4*aa*cc ) )/(2*aa);
r = min([rplus,rminus]);

%Determine circle centre
e = A + B*r; f = C + D*r;
x = e + x1; y = f + y1;

%End of code